-- Abracadabra
-- A digital healing device inspired by
-- https://en.m.wikipedia.org/wiki/Abracadabra#history
local vshader = [[
uniform mat4 modelViewProjection;
attribute vec4 position;
attribute vec4 color;
attribute vec2 texCoord;
varying lowp vec4 vColor;
varying highp vec2 vTexCoord;

void main()
{
  vColor = color;
  vTexCoord = texCoord;
  gl_Position = modelViewProjection * position;
}
]]

local fshader = [[
precision highp float;
uniform lowp sampler2D texture;
varying lowp vec4 vColor;
varying highp vec2 vTexCoord;
uniform highp vec2 offset;

void main()
{
  float row = floor(vTexCoord.y + offset.y);
  highp vec2 p = mod(vTexCoord + vec2(row*-.05, 0.) + offset, 1.);
  lowp vec4 col = texture2D( texture, p ) * vColor;
  gl_FragColor = col;
}
]]

function mix(a, b, amount) 
  return (1-amount)*a + amount*b
end

function setup()
  viewer.mode = FULLSCREEN
  m = mesh()
  offset = vec2(3.5,0)

  m:addRect(0,0,WIDTH,HEIGHT)
  m.texture = readImage(asset.pixelabra)
  m.shader = shader(vshader, fshader)
  m.shader.offset = offset
  m:setRectTex(1,0,0,560/WIDTH,20)
  rpos = vec2(1,5)
end

function draw()
  noSmooth()
  translate(WIDTH/2, HEIGHT/2)
  m.shader.offset = offset
  m:draw()
  offset = mix(offset, offset + rpos, math.min(1., DeltaTime))
end

function touched(touch)
  local delta = (touch.prevPos - touch.pos)
  delta.x = delta.x * 2. / WIDTH
  delta.y = delta.y * 20. / HEIGHT
  if touch.state == CHANGED then
    rpos = vec2(0,0)
    offset = offset + delta
  elseif touch.state == ENDED then
    rpos = delta * 20
  end
end
